## make geneset gmt file ##
## input file path /mnt/data/hong/2022/DHJ1_human_obesity_placenta/output/DEGs/summary_Cscore ##
## input file format with field index and genes ##
## output field "geneset" \t "description" \t "genes" ##
## geneset is the index column from input, description is the index column value + input file names without suffix .tsv, genes is the genes column, aggregated by the index column, separated by space into one record ##

import pandas as pd
import os
import mygene

def gene_symbol_to_entrez_id(gene_symbols):
    mg = mygene.MyGeneInfo()
    gene_symbols_list = gene_symbols.tolist()
    results = mg.querymany(gene_symbols_list, scopes='symbol', fields='entrezgene', species='human')
    df_results = pd.DataFrame(results)
    return df_results

def mk_gmt(df, descript):
    ## drop na any, i.e., the rows that contains any nan
    df = df.dropna(subset=["index", "genes"])
    ## df aggreagate by "index" and genes to a str joined by space
    ## convert genes to entrez id
    entrez_df = gene_symbol_to_entrez_id(df["genes"])
    ## merge with df
    df = pd.merge(df, entrez_df, left_on="genes", right_on="query", how="left")
    ## drop the original genes column
    df = df.drop(columns=["genes"])
    ## rename the entrezgene column to genes
    df = df.rename(columns={"entrezgene": "genes"})
    df = df.dropna(subset=["index", "genes"])
    gmt_df = df.groupby("index")["genes"].apply(lambda x: " ".join(x)).reset_index()
    ## add description
    gmt_df["description"] = gmt_df["index"] + "_" + descript
    ## add geneset
    gmt_df["geneset"] = gmt_df["index"] + "_" + descript
    ## select columns
    gmt_df = gmt_df[["geneset", "description", "genes"]]
    return gmt_df

if __name__ == "__main__":
    ## read all files in the input path
    files = os.listdir("/mnt/data/hong/2022/DHJ1_human_obesity_placenta/output/DEGs/summary_Cscore")
    gmt_dfs = []
    ## read each file
    for file in files:
        df = pd.read_csv(f"/mnt/data/hong/2022/DHJ1_human_obesity_placenta/output/DEGs/summary_Cscore/{file}", sep="\t")
        descript = file.split(".")[0]
        ## make gmt
        gmt_dfs.append(mk_gmt(df, descript))
    ## pandas concat
    gmt = pd.concat(gmt_dfs)
    ## output
    gmt.to_csv("genese_obesity_placenta.gmt", sep="\t", index=False)


import gzip
import os
import pandas as pd
import shutil
import logging

root_folder = '/mnt/data/hong/2022/DHJ1_human_obesity_placenta/data/gwas/egg-consortium.org'
## set logging path, and logging level
logging.basicConfig(filename=f'{root_folder}/mk_db.log', level=logging.DEBUG)
target_folder = '/mnt/storage/hong/2024/egg-consortium'

p = ['p', 'p-value', 'pval', 'p.value', 'p_value']
n=['n', 'nsamples', 'totalsamplesize']
rsid=['hm_rsid', 'rs_id','rsid','snp', 'markername', 'variant_id']
chromosome=['chromosome', 'chrom', 'chromosome_name', 'chr']
position=['position', 'pos', 'bp', 'basepair', 'base_pair_location', 'base_pair_location_start']


def unzip_file(file_path, target_folder):
    """unzip the gz file and move the gz file to a target folder"""
    file_name = file_path.replace('.gz', '')
    with gzip.open(file_path,"rb") as f_in, open(file_name,"wb") as f_out:
        shutil.copyfileobj(f_in, f_out)
        ## move the zip file to a target folder
        shutil.move(file_path, os.path.join(target_folder, file_path.split("/")[-1]))

def mk_pval_file(df, p, n, rsid, filename):
    """
    return a file with fields rs_id, p, and n.
    """
    col_names = df.columns.tolist()
    rsid_cols = [col for col in col_names if col.lower().strip() in rsid]
    p_cols = [col for col in col_names if col.lower().strip() in p]
    n_cols = [col for col in col_names if col.lower().strip() in n]
    ## check if the rsid_cols, p_cols, and n_cols are unique
    if len(rsid_cols) != 1 or len(p_cols) != 1:
        ## not raise error, but print a warning and continue
        logging.warning(f"Warning: There should be only one rsid_col, p_col, and n_col")
        ## -[] what if not unique?
        return
    ## get the column names
    rsid_col = rsid_cols[0]
    p_col = p_cols[0]
    ## if n_cols is empty, use 42212
    ## simplified ifelse statement
    n_col = n_cols[0] if len(n_cols) > 0 else n
    ## create df2use with the column names ['rsid', 'p', 'n']
    df2use = pd.DataFrame({"rsid": df[rsid_col], "p": df[p_col], "n": df[n_col]})
    df2use['p'] = pd.to_numeric(df2use['p'], errors='coerce')
    df2use['n'] = pd.to_numeric(df2use['n'], errors='coerce')
    ## remove any nan row
    df2use = df2use[~df2use.isna().any(axis=1)]
    ## specify the column types using pandas for dataframe but not per column
    df2use = df2use.astype({'rsid': 'str', 'p': 'float', 'n': 'int'})
    ## write the file
    filename_new = f'{filename}_p.txt'
    df2use.to_csv(filename_new, index=False, sep='\t')

## for os walk 
# -[] maybe modulize to handle the breaks of the chain of programs

def mk_loc_file(df, rsid,  chromosome, position, filename):
    """
    return a file with fields rs_id, chromosome,position.
    """
    ## get the column names
    col_names = df.columns.tolist()
    rsid_cols = [col for col in col_names if col.lower().strip() in rsid]
    chromosome_cols = [col for col in col_names if col.lower().strip() in chromosome]
    ## using any(substring in string for substring in substring_list)
    position_cols = [col for col in col_names if col.lower().strip() in position]
    if len(rsid_cols) != 1 or len(chromosome_cols) != 1 or len(position_cols) != 1:
        ## not raise error, but print a warning and continue
        logging.warning(f"Warning: we found {len(rsid_cols)} rsid_col, {len(chromosome_cols)} chromosome_col, and {len(position_cols)} position_col")
        ## -[] what if not unique?
        return
    ## get the column names
    rsid_col = rsid_cols[0]
    chromosome_col = chromosome_cols[0]
    position_col = position_cols[0]
    ## create df2use with the column names ['rsid', 'chromosome', 'position']
    df2use = pd.DataFrame({"rsid": df[rsid_col], "chromosome": df[chromosome_col], "position": df[position_col]})
    ## coerce conversion for each column, pd.to_numeric, then astype
    df2use['chromosome'] = pd.to_numeric(df2use['chromosome'], errors='coerce')
    df2use['position'] = pd.to_numeric(df2use['position'], errors='coerce')
    ## remove any nan row
    df2use = df2use[~df2use.isna().any(axis=1)]
    df2use = df2use.astype({'rsid': 'str', 'chromosome': 'int', 'position': 'int'})
    filename_new = f'{filename}_loc.txt'
    df2use.to_csv(filename_new, index=False, sep='\t')

def run_single_gwas(root, file, target_folder, p, n, rsid, chromosome, position):
    unzip_file(os.path.join(root, file), target_folder)
    file_path = os.path.join(root, file.replace('.gz', ''))
    df = pd.read_csv(file_path, sep='\t')
    filename = file_path.split("/")[-1]
    mk_pval_file(df, p, n, rsid, filename)
    mk_loc_file(df, rsid,  chromosome, position, filename)
    ## remove the unzipped file
    os.remove(os.path.join(root, file.replace('.gz', '')))

if __name__ == "__main__":
    for root, dirs, files in os.walk(root_folder):
        for file in files:
            if file.endswith('.gz'):
                ## logging
                logging.info(f"Processing {os.path.join(root, file)}")
                ## these are defaults
                run_single_gwas(root, file, target_folder, p = p, n=n, rsid=rsid, chromosome=chromosome, position=position)


import pandas as pd
import gseapy as gp
import matplotlib.pyplot as plt
from collections import defaultdict
from gseapy import barplot, dotplot
import os

wd = '/mnt/data/hong/2023/DHJ4_human_pcos_placenta/primeseq'
os.chdir(wd)

gene_sets=['MSigDB_Hallmark_2020','KEGG_2021_Human', 'GO_Biological_Process_2025', 'Reactome_Pathways_2024', 'Jensen_COMPARTMENTS', 'Human_Phenotype_Ontology', 'DGIdb_Drug_Targets_2024']
## from the intersect output get the shared across all
## get DEG list from file

def _plot(genelist, comparison, pattern):
    print(f"enrichment start {comparison} {pattern}")
    enr_list = []
    ## separately enrich and rbind
    for gset in gene_sets:
        try:
            enr = gp.enrichr(gene_list=genelist, # or "./tests/data/gene_list.txt",
                gene_sets=gset,
                organism='human', # don't forget to set organism to the one you desired! e.g. Yeast
                outdir='output/DEGs/enrichr', # don't write to disk
            )
            enr_list.append(enr.results)
            print(f"enrichment suscess {comparison} up {gset}")
        except:
            enr_list = enr_list
    if len(enr_list)>0:
        enr_pd = pd.concat(enr_list, ignore_index=True)
        barplot(enr_pd,
              column="Adjusted P-value",
              group='Gene_set', # set group, so you could do a multi-sample/library comparsion
              size=10,
              top_term=5,
              figsize=(3,5),
              ofname = f'figures/enrichr/{comparison}_{pattern}_snRNAseq.pdf',
              color = {'MSigDB_Hallmark_2020':'#4C72B0', 'KEGG_2021_Human': '#DD8452',
             'GO_Biological_Process_2023': '#55A868', 'Reactome_Pathways_2024': '#C44E52', 'Jensen_COMPARTMENTS': '#8172B2', 'Human_Phenotype_Ontology': '#9370DB', 'DGIdb_Drug_Targets_2024': '#8A3200'})
        enr_pd.to_csv(f'output/DEGs/enrichr/{comparison}_{pattern}_snRNAseq.tsv', sep='\t')

def enrich_plots(comparison, file):
    # data = pd.read_csv(file, sep=';', decimal=',')
    data = pd.read_csv(file, sep=',', decimal='.')
    cols = ["padj", "log2FoldChange"]
    data[cols] = data[cols].apply(pd.to_numeric, errors='coerce')
    genelists = defaultdict(list)
    data_sig = data.query("padj<0.05")
    print(data.head())
    genelists['up'],  genelists['down'] = data_sig.query("log2FoldChange>1")["Unnamed: 0"].to_list(), data_sig.query("log2FoldChange<(-1)")["Unnamed: 0"].to_list()
    for k,v in genelists.items():
        print(len(v))
        _plot(v, comparison, k)
    

if __name__ == "__main__":
    comparison = "placebo_control"
    file = f"output/DEGs/up_{comparison}_pc_snRNAseq.csv"
    enrich_plots(comparison, file)
    file = f"output/DEGs/down_{comparison}_pc_snRNAseq.csv"
    enrich_plots(comparison, file)


import networkx as nx
import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
import matplotlib.colors as mcolors
#import community as community_louvain

def remove_edge(G, threshold):
    long_edges = list(filter(lambda e: e[2] > threshold, (e for e in G.edges.data('correlation'))))
    le_ids = list(e[:2] for e in long_edges)
    # remove filtered edges from graph G
    G.remove_edges_from(le_ids)
    return G   

def wgcna2nx(f, unwanted, threshold):
    wgcna_data = pd.read_csv(f, sep='\t')
    tmp = pd.DataFrame(np.sort(wgcna_data[['#node1','node2']].values,1))
    tmp['correlation']=wgcna_data.correlation
    nx_data = tmp.drop_duplicates(subset=[0,1])
    network = nx.from_pandas_edgelist(nx_data,0,1,edge_attr='correlation')
    try:
        network.remove_node(unwanted)
    except Exception:
        pass
    network = remove_edge(network, threshold)
    return network 

def degree_analysis(G, name):
    degree_sequence = sorted([d for n, d in G.degree(weight='combined_score')], reverse=True)
    dmax = max(degree_sequence)

    fig = plt.figure("{}".format(name), figsize=(8, 8))
    # Create a gridspec for adding subplots of different sizes
    axgrid = fig.add_gridspec(5, 4)

    ax0 = fig.add_subplot(axgrid[0:3, :])
    Gcc = G.subgraph(sorted(nx.connected_components(G), key=len, reverse=True)[0])
    pos = nx.spring_layout(Gcc, seed=10396953)
    nx.draw_networkx_nodes(Gcc, pos, ax=ax0, node_size=20)
    nx.draw_networkx_edges(Gcc, pos, ax=ax0, alpha=0.4)
    ax0.set_title("Connected components of {}".format(name))
    ax0.set_axis_off()

    ax1 = fig.add_subplot(axgrid[3:, :2])
    ax1.plot(degree_sequence, "b-", marker="o")
    ax1.set_title("Degree Rank Plot")
    ax1.set_ylabel("Degree")
    ax1.set_xlabel("Rank")

    ax2 = fig.add_subplot(axgrid[3:, 2:])
    ax2.bar(*np.unique(degree_sequence, return_counts=True))
    ax2.set_title("Degree histogram")
    ax2.set_xlabel("Degree")
    ax2.set_ylabel("# of Nodes")

    fig.tight_layout()
    plt.savefig('figures/{}_degree.pdf'.format(name))

def remove_low_degree(G,dthres):
    low_degree = [n for n, d in G.degree() if d < dthres]
    return G.remove_nodes_from(low_degree)

def betweeness_analysis(G,name):
    # largest connected component
    components = nx.connected_components(G)
    largest_component = max(components, key=len)
    H = G.subgraph(largest_component)

    # compute centrality
    centrality = nx.betweenness_centrality(H, k=10, endpoints=True)

    # compute community structure
    lpc = nx.community.label_propagation_communities(H)
    community_index = {n: i for i, com in enumerate(lpc) for n in com}

    #### draw graph ####
    fig, ax = plt.subplots(figsize=(20, 15))
    pos = nx.spring_layout(H, k=0.15, seed=4572321)
    node_color = [community_index[n] for n in H]
    node_size = [v * 20000 for v in centrality.values()]
    largest = max(centrality, key=centrality.get)
    second = {i:centrality[i] for i in centrality if i!=largest}
    second_largest = max(second, key=second.get)
    x1, y1 = pos[largest]
    x2, y2 = pos[second_largest]
    plt.text(x1, y1, largest)
    plt.text(x2, y2, second_largest)
#     if name == 'HFHS':
#         third = {i:second[i] for i in second if i!=second_largest}
#         third_largest = max(third, key=third.get)
#         x3, y3 = pos[third_largest]
#         plt.text(x3, y3, third_largest)
    nx.draw_networkx(
        H,
        pos=pos,
        with_labels=False,
        node_color=node_color,
        node_size=node_size,
        edge_color="gainsboro",
        alpha=0.4,
    )

    # Title/legend
    font = {"color": "k", "fontweight": "bold", "fontsize": 20}
    ax.set_title("Metabolic network for DEG in {}".format(name), font)
    # Change font color for legend
    font["color"] = "r"

    ax.text(
        0.80,
        0.10,
        "node color = community structure",
        horizontalalignment="center",
        transform=ax.transAxes,
        fontdict=font,
    )
    ax.text(
        0.80,
        0.06,
        "node size = betweeness centrality",
        horizontalalignment="center",
        transform=ax.transAxes,
        fontdict=font,
    )

    # Resize figure for label readibility
    ax.margins(0.1, 0.05)
    fig.tight_layout()
    plt.axis("off")
    plt.savefig('figures/{}_betweeness_annotate.pdf'.format(name))

def bubble(G,name,degre_threshold, color=None, community_colored=False, text_annot=False, gene_list=False, specify_color=False, size_dict=None):
    # largest connected component
    components = nx.connected_components(G)
    largest_component = max(components, key=len)
    H = G.subgraph(largest_component)
    
    ## full network
    # H = G

    # compute centrality
    centrality = dict(H.degree(weight='combined_score'))

    #### draw graph ####
    
    pos = nx.spring_layout(H, k=0.15, seed=4572321)
    if size_dict is None:
        node_size = np.array([v for v in centrality.values()])
        node_size *= 5000.0/node_size.max()
    else:
        node_size = []
        for node in H:
            try:
                node_size.append(size_dict[node]*10)
            except:
                node_size.append(0)
    if community_colored == True:
        community_index = community(H, weight='combined_score')
        community_data = pd.DataFrame.from_dict(community_index, orient='index')
        community_data.to_csv(f'{name}_community.tsv', sep='\t', header=False, index=True)
        node_color = [community_index[n] for n in H]
    if specify_color:
        node_color = []
        for node in H:
            node_color.append(specify_color[node])
    else:
        node_color = np.where(np.fromiter(centrality.values(), dtype=float) > degre_threshold, color, mcolors.TABLEAU_COLORS['tab:gray'])
    #node_color = np.where(np.isin(np.array([a for a in H.nodes()]), np.array(gene_list)), color, mcolors.TABLEAU_COLORS['tab:gray'])
    if not gene_list:
        genes = sorted(centrality, key=centrality.get, reverse=True)
    else:
        genes = gene_list
    draw_params(H, pos, node_color, node_size, genes, color, name, text_annot=text_annot)

def community(H, weight):
    lpc = nx.community.louvain_communities(H, weight)
    community_index = {n: i for i, com in enumerate(lpc) for n in com}
    return community_index

    
def draw_params(H, pos, node_color, node_size, dict_sorted, color, name, text_annot):
    fig, ax = plt.subplots(figsize=(20, 15))
    nx.draw_networkx(
        H,
        pos=pos,
        cmap=plt.get_cmap('coolwarm'),
        with_labels=False,
        node_color=node_color,
        node_size=node_size,
        edge_color="gainsboro",
        alpha=0.8,
        vmin=-8,
        vmax=8
    )
    sm = plt.cm.ScalarMappable(cmap=plt.get_cmap('coolwarm'), norm=plt.Normalize(vmin = -8, vmax=8))
    sm._A = []
    plt.colorbar(sm, ax=ax)
    if text_annot:
        for i in dict_sorted:
            x, y = pos[i]
            plt.text(x, y, i, fontsize=24, color=color)
    # x, y = pos[gene_list]
    # plt.text(x, y, gene_list, fontsize=18, color='r')

    # Title/legend
    font = {"color": "k", "fontweight": "bold", "fontsize": 20}
    #ax.set_title("Metabolic network for DEG in {}".format(name), font)
    # Change font color for legend
    font["color"] = color

    ax.text(
        0.80,
        0.06,
        "node size = perturbed target gene number",
        horizontalalignment="center",
        transform=ax.transAxes,
        fontdict=font,
    )

    # Resize figure for label readibility
    ax.margins(0.1, 0.05)
    fig.tight_layout()
    plt.axis("off")
    plt.savefig('figures/{}_bubble_network.pdf'.format(name))
